package	jp.co.sds_corp.nanchart.graph;

import java.io.Serializable;
import java.math.BigDecimal;

import jp.co.sds_corp.util.common.CmnDate;

/**
Valueクラスは、データの値をカプセル化します。
*/
public class Value implements Comparable<Object>, Cloneable, Serializable
{
	private static final long serialVersionUID = 1L;

	/** 型：数値 */
	public final static int	NUMBER		= 0;
	/** 型：Integer */
	public final static int	INT			= 11;
	/** 型：Byte */
	public final static int	BYTE		= 12;
	/** 型：Char */
	public final static int	CHAR		= 13;
	/** 型：Double */
	public final static int	DOUBLE		= 14;
	/** 型：Float */
	public final static int	FLOAT		= 15;
	/** 型：文字列 */
	public final static int	STRING		= 20;
	/** 型：日付 */
	public final static int	DATE		= 21;
	/** 型：数値 */
	public final static int	DECIMAL		= 22;

	/** 変数のタイプ */
	int		nType = -1;
	/** 変数の値 */
	Object	value = null;

	/**
	* int引数を表す、新たなValueオブジェクトを構築します。
	* @param	val		値
	*/
	public Value( int val ) {
		nType = INT;
		value = Integer.valueOf( val );
	}

	/**
	* double引数を表す、新たなValueオブジェクトを構築します。
	* @param	val		値
	*/
	public Value( double val ) {
		nType = DOUBLE;
		value = Double.valueOf( val );
	}

	/**
	* Object引数を表す、新たなValueオブジェクトを構築します。
	* @param	val		値
	*/
	public Value( Object val ) {
		value = val;
		nType = getType();
	}

	/**
	* 設定する値を取得します。
	* @return	値
	*/
	public Object getObject() {
		return value;
	}

	/**
	* このValueをほかのObjectと比較し、結果をdoubleで返します。
	* 比較できるのは同じクラスのオブジェクト同士でなくてはなりません。
	* 例外としては、{@link Number}クラスのサブクラス同士の演算は可能です。
	* 現在の所{@link Number}クラス以外では、{@link CmnDate}クラスのみを
	* サポートしています。
	* @param	obj		比較するオブジェクト
	* @return	演算結果
	* @throws	ClassCastException	obj同士の型が合わない場合
	* @see		#compareTo(Object)
	*/
	public double diff( Object obj ) {
		String	vn = value.getClass().getName();
		Object	sub = obj;
		if ( sub instanceof Value ) {
			sub = ((Value)obj).getObject();
		}
		String	on = sub.getClass().getName();

		if (( value instanceof Number ) && ( sub instanceof Number )) {
		} else {
			if ( !vn.equals( on ) ) {
				throw( new ClassCastException( on + " must be " + vn ));
			}
		}

		double	dRet = Double.NaN;
		switch ( nType ) {
		case NUMBER :
		case INT :
		case BYTE :
		case DOUBLE :
		case FLOAT :
		case DECIMAL :
			dRet = (double )(((Number)value).doubleValue()
							- ((Number)sub).doubleValue() );
			break;
		case DATE :
//			dRet = (double )(((CmnDate)value).diffDays( (CmnDate)sub ));
			dRet = (double )(((CmnDate)value).getTime().getTime()
							- ((CmnDate)sub).getTime().getTime() );
			break;
		case CHAR :
		case STRING :
			throw( new IllegalArgumentException( "cannot calc " + vn ));
		}
		return dRet;
	}

	/**
	* このValueをほかのObjectと比較し、結果をintで返します。
	* @return	このValueの値がobjより小さい場合は負の数、等しい場合は 0、
	*			大きい場合は正の数。
	* @throws	ClassCastException	obj同士の型が合わない場合
	* @see		#diff(Object)
	*/
	public int compareTo( Object obj ) {
		return (int )diff( obj );
	}

	/**
	* Valueに設定されている値の型を返します。
	* @return	値の型
	* @see		#getType(Object)
	* @see		#NUMBER
	* @see		#INT
	* @see		#BYTE
	* @see		#CHAR
	* @see		#DOUBLE
	* @see		#FLOAT
	* @see		#STRING
	* @see		#DATE
	* @see		#DECIMAL
	*/	
	public int getType() {
		return getType( this.value );
	}

	/**
	* valueの型を返します。
	* @return	値の型
	* @see		#getType()
	* @see		#NUMBER
	* @see		#INT
	* @see		#BYTE
	* @see		#CHAR
	* @see		#DOUBLE
	* @see		#FLOAT
	* @see		#STRING
	* @see		#DATE
	* @see		#DECIMAL
	*/	
	public static int getType( Object value ) {
		int		nRet = -1;
		if ( value instanceof Integer ) {
			nRet = INT;
		} else if ( value instanceof Byte ) {
			nRet = BYTE;
		} else if ( value instanceof Double ) {
			nRet = DOUBLE;
		} else if ( value instanceof Float ) {
			nRet = FLOAT;
		} else if ( value instanceof Number ) {
			nRet = NUMBER;
		} else if ( value instanceof Character ) {
			nRet = CHAR;
		} else if ( value instanceof String ) {
			nRet = STRING;
		} else if ( value instanceof CmnDate ) {
			nRet = DATE;
		} else if ( value instanceof BigDecimal ) {
			nRet = DECIMAL;
		}
		return nRet;
	}

	/**
	* 指定された型の最小値を返します。
	* @param	nType	最小値を求める型
	* @return	最小値
	*/
	public static Object getMinValue( int nType ) {
		Object ret = null;
		switch ( nType ) {
		case INT :		ret = Integer.valueOf( Integer.MIN_VALUE ); break;
		case BYTE :		ret = Byte.valueOf( Byte.MIN_VALUE ); break;
		case DOUBLE :	ret = Double.valueOf( Double.NEGATIVE_INFINITY ); break;
		case FLOAT :	ret = Float.valueOf( Float.NEGATIVE_INFINITY ); break;
		case DECIMAL :	ret = new BigDecimal( 0 - Double.MAX_VALUE ); break;
		case DATE :
			try {
				ret = new CmnDate( "19700101" );
			} catch ( Exception e ) {
				ret = null;
			}
			break;
		case CHAR :
		case STRING :
			throw( new IllegalArgumentException( "cannot calc " ));
		}
		return ret;
	}

	/**
	* 指定された型の最大値を返します。
	* @param	nType	最大値を求める型
	* @return	最大値
	*/
	public static Object getMaxValue( int nType ) {
		Object ret = null;
		switch ( nType ) {
		case INT :		ret = Integer.valueOf( Integer.MAX_VALUE ); break;
		case BYTE :		ret = Byte.valueOf( Byte.MAX_VALUE ); break;
		case DOUBLE :	ret = Double.valueOf( Double.POSITIVE_INFINITY ); break;
		case FLOAT :	ret = Float.valueOf( Float.POSITIVE_INFINITY ); break;
		case DECIMAL :	ret = new BigDecimal( Double.POSITIVE_INFINITY ); break;
		case DATE :
			try {
				ret = new CmnDate( "29991231" );
			} catch ( Exception e ) {
				ret = null;
			}
			break;
		case CHAR :
		case STRING :
			throw( new IllegalArgumentException( "cannot calc " ));
		}
		return ret;
	}

	/**
	* このオブジェクトのコピーを作成して返します。
	* @return	このインスタンスの複製
	* @throws	CloneNotSupportedException	オブジェクトのクラスが
				Cloneable インタフェースをサポートしていない場合。
	* @throws	OutOfMemoryError	十分なメモリがない場合。
	*/
	public Object clone()
			throws CloneNotSupportedException, OutOfMemoryError
	{
		return super.clone();
	}
}

